@@ -1,8 +1,5 @@
 package Imager::Font::BBox;
 use strict;
-use vars qw($VERSION);
-
-$VERSION = sprintf "%d.%03d", q$Revision: 1.5 $=~/\d+/g;
 
 =head1 NAME
 
@@ -18,8 +15,6 @@ Imager::Font::BBox - objects representing the bounding box of a string.
 
   # methods
   my $start = $bbox->start_offset;
-  my $left_bearing = $bbox->left_bearing;
-  my $right_bearing = $bbox->right_bearing;
   my $end = $bbox->end_offset;
   my $gdescent = $box->global_descent;
   my $gascent = $bbox->global_ascent;
@@ -28,7 +23,6 @@ Imager::Font::BBox - objects representing the bounding box of a string.
   my $total_width = $bbox->total_width;
   my $fheight = $bbox->font_height;
   my $theight = $bbox->text_height;
-  my $display_width = $bbox->display_width;
 
 =head1 DESCRIPTION
 
@@ -51,8 +45,6 @@ first glyph is to the right of the drawing location.
 The alias neg_width() is present to match the bounding_box()
 documentation for list context.
 
-The alias left_bearing() is present to match font terminology.
-
 =cut
 
 sub start_offset {
@@ -63,10 +55,6 @@ sub neg_width {
   return $_[0][0];
 }
 
-sub left_bearing {
-  return $_[0][0];
-}
-
 =item end_offset()
 
 The offset from the selected drawing location to the right edge of the
@@ -135,15 +123,12 @@ sub ascent {
 
 =item advance_width()
 
-The advance width of the string, if the driver supports that,
-otherwise the same as end_offset.
-
 =cut
 
 sub advance_width {
   my $self = shift;
 
-  @$self > 6 ? $self->[6] : $self->[2];
+  @$self > 6 ? $self->[6] : $self->[5];
 }
 
 =item total_width()
@@ -181,42 +166,6 @@ sub text_height {
   $self->ascent - $self->descent;
 }
 
-=item right_bearing
-
-The distance from the right of the last glyph to the end of the advance
-point.
-
-If the glyph overflows the right side of the advance width this value
-is negative.
-
-=cut
-
-sub right_bearing {
-  my $self = shift;
-
-  @$self >= 8 && return $self->[7]; # driver gives it to us
-
-  # otherwise the closest we have is the difference between the 
-  # end_pos and advance_width
-  return $self->advance_width - $self->pos_width;
-}
-
-=item display_width
-
-The distance from the left-most pixel of the left-most glyph to the
-right-most pixel of the right-most glyph.
-
-Equals advance_width - left_bearing - right_bearing (and implemented
-that way.)
-
-=cut
-
-sub display_width {
-  my ($self) = @_;
-
-  $self->advance_width - $self->left_bearing - $self->right_bearing;
-}
-
 =back
 
 =head1 INTERNAL FUNCTIONS
