@@ -4,6 +4,8 @@ use Imager::Color;
 use vars qw(@ISA);
 @ISA = qw(Imager::Font);
 
+*_first = \&Imager::Font::_first;
+
 my $t1aa;
 
 # $T1AA is in there because for some reason (probably cache related) antialiasing
@@ -69,15 +71,20 @@ sub _draw {
   my $self = shift;
   my %input = @_;
   t1_set_aa_level($input{aa});
+  my $flags = '';
+  $flags .= 'u' if $input{underline};
+  $flags .= 's' if $input{strikethrough};
+  $flags .= 'o' if $input{overline};
   if (exists $input{channel}) {
     Imager::i_t1_cp($input{image}{IMG}, $input{'x'}, $input{'y'},
 		    $input{channel}, $self->{id}, $input{size},
-		    $input{string}, length($input{string}), $input{align});
+		    $input{string}, length($input{string}), $input{align},
+                    $input{utf8}, $flags);
   } else {
     Imager::i_t1_text($input{image}{IMG}, $input{'x'}, $input{'y'}, 
 		      $input{color}, $self->{id}, $input{size}, 
 		      $input{string}, length($input{string}), 
-		      $input{align});
+		      $input{align}, $input{utf8}, $flags);
   }
 
   return $self;
@@ -86,10 +93,47 @@ sub _draw {
 sub _bounding_box {
   my $self = shift;
   my %input = @_;
+  my $flags = '';
+  $flags .= 'u' if $input{underline};
+  $flags .= 's' if $input{strikethrough};
+  $flags .= 'o' if $input{overline};
   return Imager::i_t1_bbox($self->{id}, $input{size}, $input{string},
-			   length($input{string}));
+			   length($input{string}), $input{utf8}, $flags);
 }
 
+# check if the font has the characters in the given string
+sub has_chars {
+  my ($self, %hsh) = @_;
+
+  unless (defined $hsh{string} && length $hsh{string}) {
+    $Imager::ERRSTR = "No string supplied to \$font->has_chars()";
+    return;
+  }
+  return Imager::i_t1_has_chars($self->{id}, $hsh{string}, $hsh{'utf8'} || 0);
+}
+
+sub utf8 {
+  1;
+}
+
+sub face_name {
+  my ($self) = @_;
+
+  Imager::i_t1_face_name($self->{id});
+}
+
+sub glyph_names {
+  my ($self, %input) = @_;
+
+  my $string = $input{string};
+  defined $string
+    or return Imager->_seterror("no string parameter passed to glyph_names");
+  my $utf8 = _first($input{utf8} || 0);
+
+  Imager::i_t1_glyph_name($self->{id}, $string, $utf8);
+}
+
+
 1;
 
 __END__
@@ -113,6 +157,28 @@ can re-enable that by calling Imager::init() with the C<t1log> option:
 
   Imager::init(t1log=>1);
 
+Currently specific to Imager::Font::Type1, you can use the following
+flags when drawing text or calculating a bounding box:
+
+=over
+
+=item underline
+
+Draw the text with an underline.
+
+=item overline
+
+Draw the text with an overline.
+
+=item strikethrough
+
+Draw the text with a strikethrough.
+
+=back
+
+Obviously, if you're calculating the bounding box the size of the line
+is included in the box, and the line isn't drawn :)
+
 =head1 AUTHOR
 
 Addi, Tony
